#!/usr/bin/env python3

import hashlib
import os
import sys

from urllib.request import urlretrieve

DSC_FILE = "deb/golang.dsc"
DSC_FILE_TEMPLATE = "templates/golang.dsc.template"

SPEC_FILE = "rpm/golang.spec"
SPEC_FILE_TEMPLATE = "templates/golang.spec.template"

BUF_SIZE = 65536

if __name__ == "__main__":

    # Check the number of arguments in the command line
    if len(sys.argv) != 3:
        print("Please give at least 2 argument: update_version.py <dl_url> <version>")
        sys.exit(1)
    
    # Get the URL
    download_url = sys.argv[1]
    # Get the version
    new_version = sys.argv[2]

    # Download locally the archive
    filename = download_url.rsplit('/', 1)[-1]
    print(f"Downloading locally the archive named {filename}...")
    urlretrieve(download_url, filename)

    # Calculate the SHA1, SHA256 and MD5SUM
    md5 = hashlib.md5()
    sha1 = hashlib.sha1()
    sha256 = hashlib.sha256()

    with open(filename, 'rb') as f:
        while True:
            data = f.read(BUF_SIZE)
            if not data:
                break
            md5.update(data)
            sha1.update(data)
            sha256.update(data)

    # Display the various values found
    md5_value = md5.hexdigest()
    sha1_value = sha1.hexdigest()
    sha256_value = sha256.hexdigest()
    size_value = os.path.getsize(filename)
    print()
    print(f"SHA1 FOUND {sha1_value}")
    print(f"SHA256 FOUND {sha256_value}")
    print(f"MD5 FOUND {md5_value}")
    print(f"FILE SIZE FOUND {size_value}")
    print()

    ###### DEBIAN DSC PART ######
    with open(DSC_FILE_TEMPLATE) as f:
        dsc_template = f.read()
    
    # Now replace the different values
    dsc_template = dsc_template.replace("%VERSION%", new_version)
    dsc_template = dsc_template.replace("%ARCHIVE_SIZE%", str(size_value))
    dsc_template = dsc_template.replace("%ARCHIVE_NAME%", filename)
    dsc_template = dsc_template.replace("%ARCHIVE_SHA1%", sha1_value)
    dsc_template = dsc_template.replace("%ARCHIVE_SHA256%", sha256_value)
    dsc_template = dsc_template.replace("%ARCHIVE_MD5SUM%", md5_value)

    # Now overwrite the golang.dsc file
    with open(DSC_FILE, 'w') as f:
        f.write(dsc_template)

    ###### SPECFILE PART ######
    with open(SPEC_FILE_TEMPLATE) as f:
        spec_template = f.read()

    # Now replace the version values
    spec_template = spec_template.replace("%VERSION%", new_version)

    # Now overwrite the golang.spec file
    with open(SPEC_FILE, 'w') as f:
        f.write(spec_template)

    print(f"!!!!!!!!! DO NOT FORGET TO UPDATE THE _service FILE IN OBS WITH THE RIGHT URL !!!!!!!!!")
    print(f"!!!!!!!!! DO NOT FORGET TO TAG THE golang-binary-packaging REPO WITH THE NEW VERSION !!!!!!!!!")
